<?php

namespace FiloBlu\CustomerBlocker\Plugin\Adyen\Payment\Model;

use Magento\Framework\ObjectManagerInterface;
use FiloBlu\CustomerBlocker\Helper\DataHelper;

class AdyenPaymentProcess
{
    /**
     * @var ObjectManagerInterface
     */
    protected $_objectManager;

    /**
     * @var DataHelper
     */
    protected $helper;

    /**
     * AdyenPaymentProcess constructor.
     * @param ObjectManagerInterface $_objectManager
     */
    public function __construct(
        ObjectManagerInterface $_objectManager,
        DataHelper $helper
    )
    {
        $this->_objectManager = $_objectManager;
        $this->helper = $helper;
    }

    /**
     *
     * This plugin is used to prevent the transaction from being created on Adyen when the order has to be blocked ftom this module.
     * The plugin only works with versions of mod2-adyen less than 5.* (the problem is present only with these versions).
     *
     * @param \Adyen\Payment\Model\AdyenPaymentProcess $subject
     * @param \Closure $proceed
     * @param string $payload
     * @return bool|mixed
     */
    public function aroundInitiate(
        \Adyen\Payment\Model\AdyenPaymentProcess $subject,
        \Closure $proceed,
        $payload
    ) {
        $emailToBlock = false;
        $custormerAddressToBlock = false;

        if($this->helper->isModuleEnable())
        {
            // Decode payload from frontend
            $payloadDecode = json_decode($payload, true);
            $quoteEmail = !empty($payloadDecode['additional_data']['guestEmail']) ? $payloadDecode['additional_data']['guestEmail'] : '';
            $session = $this->_objectManager->create('\Magento\Checkout\Model\Session');
            $quoteData = $session->getQuote();
            $shippingAddr = $quoteData->getShippingAddress();
            $shippingAddrData = $this->helper->mappingAddressData($shippingAddr->getData());

            if($quoteEmail == '' || is_null($quoteEmail))
            {
                if($shippingAddr->getEmail() != '' && !is_null($shippingAddr->getEmail()))
                {
                    $quoteEmail = $shippingAddr->getEmail();
                }
                elseif($shippingAddrData['email'] != '' && !is_null($shippingAddrData['email']))
                {
                    $quoteEmail = $shippingAddrData['email'];
                }
                else
                {
                    $billingAddress = $quoteData->getBillingAddress();
                    $quoteEmail = $billingAddress->getEmail();
                }
            }

            $emailToBlock = $this->helper->isEmailToBeBlocked($quoteEmail);
            $custormerAddressToBlock = $this->helper->isAddressToBeBlocked($shippingAddrData);
        }

        if ($emailToBlock || $custormerAddressToBlock)
        {
            return false;
        }
        else
        {
            return $proceed($payload);
        }
    }
}